@@ -1,7 +1,7 @@
 package Complete::Unix;
 
-our $DATE = '2015-11-29'; # DATE
-our $VERSION = '0.06'; # VERSION
+our $DATE = '2016-10-26'; # DATE
+our $VERSION = '0.07'; # VERSION
 
 use 5.010001;
 use strict;
@@ -20,6 +20,9 @@ our @EXPORT_OK = qw(
 
                        complete_pid
                        complete_proc_name
+
+                       complete_service_name
+                       complete_service_port
                 );
 
 our %SPEC;
@@ -187,6 +190,77 @@ sub complete_proc_name {
         word=>$word);
 }
 
+$SPEC{complete_service_name} = {
+    v => 1.1,
+    summary => 'Complete from list of service names from /etc/services',
+    args => {
+        %arg_word,
+    },
+    result_naked => 1,
+    result => {
+        schema => 'array',
+    },
+};
+sub complete_service_name {
+    require Parse::Services;
+
+    my %args  = @_;
+    my $word  = $args{word} // "";
+
+    my %services;
+
+    # from /etc/services
+    {
+        my $res = Parse::Services::parse_services();
+        last if $res->[0] != 200;
+        for my $row (@{ $res->[2] }) {
+            $services{$row->{name}}++;
+            $services{$_}++ for @{$row->{aliases}};
+        }
+    }
+
+    require Complete::Util;
+    Complete::Util::complete_hash_key(
+        word => $word,
+        hash => \%services,
+    );
+}
+
+$SPEC{complete_service_port} = {
+    v => 1.1,
+    summary => 'Complete from list of service ports from /etc/services',
+    args => {
+        %arg_word,
+    },
+    result_naked => 1,
+    result => {
+        schema => 'array',
+    },
+};
+sub complete_service_port {
+    require Parse::Services;
+
+    my %args  = @_;
+    my $word  = $args{word} // "";
+
+    my %services;
+
+    # from /etc/services
+    {
+        my $res = Parse::Services::parse_services();
+        last if $res->[0] != 200;
+        for my $row (@{ $res->[2] }) {
+            $services{$row->{port}}++;
+        }
+    }
+
+    require Complete::Util;
+    Complete::Util::complete_hash_key(
+        word => $word,
+        hash => \%services,
+    );
+}
+
 1;
 # ABSTRACT: Unix-related completion routines
 
@@ -202,7 +276,7 @@ Complete::Unix - Unix-related completion routines
 
 =head1 VERSION
 
-This document describes version 0.06 of Complete::Unix (from Perl distribution Complete-Unix), released on 2015-11-29.
+This document describes version 0.07 of Complete::Unix (from Perl distribution Complete-Unix), released on 2016-10-26.
 
 =head1 DESCRIPTION
 
@@ -289,6 +363,44 @@ Word to complete.
 Return value:  (array)
 
 
+=head2 complete_service_name(%args) -> array
+
+Complete from list of service names from /etc/services.
+
+This function is not exported by default, but exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<word>* => I<str> (default: "")
+
+Word to complete.
+
+=back
+
+Return value:  (array)
+
+
+=head2 complete_service_port(%args) -> array
+
+Complete from list of service ports from /etc/services.
+
+This function is not exported by default, but exportable.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<word>* => I<str> (default: "")
+
+Word to complete.
+
+=back
+
+Return value:  (array)
+
+
 =head2 complete_uid(%args) -> array
 
 Complete from list of Unix UID's.
@@ -330,10 +442,6 @@ Word to complete.
 
 Return value:  (array)
 
-=head1 SEE ALSO
-
-L<Complete::Util>
-
 =head1 HOMEPAGE
 
 Please visit the project's homepage at L<https://metacpan.org/release/Complete-Unix>.
@@ -350,13 +458,17 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<Complete::Util>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
